/* GNOME clock & mailcheck applet
 * (C) 2003 John Ellis
 *
 * Author: John Ellis
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 
 * 02139, USA.
 *
 */

#include "clockmail.h"
#include "skin.h"
#include "update.h"
#include "display.h"

static void about_cb(PanelApplet *widget, gpointer data)
{
	static GtkWidget *about = NULL;
	GtkWidget *href;

	const gchar *authors[] = {
		"John Ellis <johne@bellatlantic.net>",
		NULL
		};
	const gchar *documenters[] = {
		NULL
		};
	const gchar *translators = _("translators");

	if (about != NULL)
		{
		gdk_window_show(about->window);
		gdk_window_raise(about->window);
		return;
		}

	about = gnome_about_new ( _("Sound Monitor Applet"), VERSION,
			"(C) 2003 John Ellis",
			_("The GNOME clock and mailcheck applet."),
			authors, documenters,
			strcmp (translators, "translators") != 0 ? translators : NULL,
			NULL);

	href = gnome_href_new("http://gqapplets.sourceforge.net", _("Clockmail home page"));
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(about)->vbox), href, FALSE, FALSE, 0);
	gtk_widget_show(href);

	gtk_window_set_wmclass(GTK_WINDOW(about), "clockmail", "Clockmail");

	gtk_signal_connect(GTK_OBJECT(about), "destroy",
			   GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about);
	gtk_widget_show(about);
}

void launch_mail_reader(gpointer data)
{
	AppData *ad = data;
	if (ad->reader_exec_cmd && strlen(ad->reader_exec_cmd) > 0)
		{
		gnome_execute_shell(NULL, ad->reader_exec_cmd);
		}
}

/*
 * Get file modification time, based upon the code
 * of Byron C. Darrah for coolmail and reused on fvwm95
 * I updated this by adding the old/new time check so the newmail notifier works
 * the way I wanted to work (eg. newmail is true only when mail has arrived since last check)
 * added a reset command in the event the mail_file has changed from the properties dialog
 */
void check_mail_file_status (int reset, AppData *ad)
{
	struct stat s;
	off_t newsize;
	time_t newtime;
	int status;
	int checktime;

	if (reset)
		{
		ad->mailsize = 0;
		ad->oldtime = 0;
		return;
		}

	/* no need to continue if no mail item is available */
	if (!ad->skin->mail && !ad->skin->mail_count) return;

	/* check if mail file contains something */
	if (!ad->mail_file) return;

	status = stat (ad->mail_file, &s);
	if (status < 0)
		{
		ad->mailsize = 0;
		ad->anymail = ad->newmail = ad->unreadmail = 0;
		return;
		}
	
	newsize = s.st_size;
	newtime = s.st_mtime;
	ad->anymail = newsize > 0;
	ad->unreadmail = (s.st_mtime >= s.st_atime && newsize > 0);

	checktime = (newtime > ad->oldtime);
	if (newsize > ad->mailsize && ad->unreadmail && checktime)
		{
		ad->newmail = 1;
		ad->mailcleared = 0;
		}
	else
		{
		ad->newmail = 0;
		}
	ad->oldtime = newtime;
	ad->mailsize = newsize;
}

void set_tooltip(struct tm *time_data, AppData *ad)
{
	if (time_data->tm_yday != ad->old_yday)
		{
		gchar date[256];
		gchar *utf8;

		strftime(date, sizeof(date), _("%A, %B %d"), time_data);
		utf8 = g_locale_to_utf8(date, -1, NULL, NULL, NULL);

		if (ad->use_gmt)
			{
			gchar *buf;
			if (ad->gmt_offset == 0)
				{
				buf = g_strconcat (utf8, _(" (GMT)"), NULL);
				}
			else
				{
				gchar gmt_text[32];
				g_snprintf(gmt_text, sizeof(gmt_text), 
					   _(" (GMT %+d)"), ad->gmt_offset);
				buf = g_strconcat (utf8, gmt_text, NULL);
				}
			gtk_tooltips_set_tip(ad->tooltips, GTK_WIDGET(ad->applet), buf, NULL);
			g_free(buf);
			}
		else
			{
			gtk_tooltips_set_tip(ad->tooltips, GTK_WIDGET(ad->applet), utf8, NULL);
			}
		g_free(utf8);

		ad->old_yday = time_data->tm_yday;
		}
}

void reload_skin(AppData *ad)
{
	if (ad->theme_file && strlen(ad->theme_file) == 0)
		{
		skin_set(NULL, ad);
		}
	else if (!skin_set(ad->theme_file, ad))
		{
		printf("Failed to load skin %s, loading default\n", ad->theme_file);
		skin_set(NULL, ad);
		}
}

/* clean up function to free all the applet's memory and stop timers */
static void destroy_applet(GtkWidget *widget, gpointer data)
{
	AppData *ad = data;

	if (ad->propwindow)
		{
		gtk_signal_disconnect_by_data(GTK_OBJECT(ad->propwindow), ad);
		gtk_widget_destroy(ad->propwindow);
		ad->propwindow = NULL;
		}

	if (ad->tooltips) g_object_unref(ad->tooltips);

	property_save();

	gtk_timeout_remove(ad->update_timeout_id);
	if (ad->blinking) gtk_timeout_remove(ad->blink_timeout_id);

	skin_free(ad->skin);

	g_free(ad->mail_file);
	g_free(ad->newmail_exec_cmd);
	g_free(ad->theme_file);
	g_free(ad);
}

static void applet_change_orient(GtkWidget *w, PanelAppletOrient o, gpointer data)
{
	AppData *ad = data;
	ad->orient = o;

	reload_skin(ad);
}

static gint size_hint_from_size(gint size)
{
	if (size < 36)
		return SIZEHINT_TINY;
	else if (size < 48)
		return SIZEHINT_SMALL;
	else if (size < 64)
		return SIZEHINT_STANDARD;
	else if (size < 80)
		return SIZEHINT_LARGE;
	else
		return SIZEHINT_HUGE;
}

static void applet_change_size(GtkWidget *w, int size, gpointer data)
{
	AppData *ad = data;

	ad->sizehint = size_hint_from_size(size);
	
	reload_skin(ad);
}

static void applet_do_draw(GtkWidget *w, PanelAppletBackgroundType  type, GdkColor *color, GdkPixmap *pixmap, gpointer data)
{
	AppData *ad = data;

	applet_skin_backing_sync(ad);
}

/* popup menu */
static const BonoboUIVerb applet_menu_verbs[] = {
	BONOBO_UI_UNSAFE_VERB("Properties", property_show),
	BONOBO_UI_UNSAFE_VERB("About", about_cb),
	BONOBO_UI_VERB_END
};


static gint applet_widget_new(PanelApplet *applet)
{
        AppData *ad;

	gnome_window_icon_set_default_from_file(GNOME_ICONDIR"/gnome-clockmail.png");

	panel_applet_add_preferences(applet, "/schemas/apps/clockmail-applet/prefs", NULL);

        ad = g_new0(AppData, 1);

	ad->applet = applet;
	ad->propwindow = NULL;
	ad->orient = panel_applet_get_orient(applet);
	ad->sizehint = size_hint_from_size(panel_applet_get_size(applet));

	ad->mailsize = 0;
	ad->oldtime = 0;
	ad->old_week = -1;
	ad->old_n = 0;
	ad->blink_lit = 0;
	ad->blink_count = 0;

	ad->use_gmt = FALSE;
	ad->gmt_offset = 0;
	ad->mail_max = 100;

	ad->message_count = 0;

	/* (duration = BLINK_DELAY / 1000 * BLINK_TIMES) */
	ad->blink_delay = 166;
	ad->blink_times = 16;

	ad->am_pm_enable = FALSE;
	ad->always_blink = FALSE;

	/* mail file location */
	ad->mail_file = NULL;

	/* execute a command on button press */
	ad->reader_exec_cmd = NULL;

	/* execute a command on new mail */
	ad->newmail_exec_cmd = NULL;
	ad->exec_cmd_on_newmail = FALSE;

	ad->skin = NULL;

	ad->theme_file = NULL;

	property_load(ad);

	/* get mail filename from environment if not specified in the session file */
	if (!ad->mail_file || strlen(ad->mail_file) == 0)
		{
		g_free(ad->mail_file);
		ad->mail_file = NULL;
		if (getenv ("MAIL"))
		       	{
			ad->mail_file = g_strdup(getenv ("MAIL"));
			}
		else if (getenv("USER"))
			{
			ad->mail_file = g_strconcat("/var/spool/mail/", getenv("USER"), NULL);
			}
		}

	ad->display = gtk_drawing_area_new();
	gtk_signal_connect(GTK_OBJECT(ad->display), "destroy",
			   GTK_SIGNAL_FUNC(destroy_applet), ad);
	gtk_widget_show(ad->display);

	gtk_container_add(GTK_CONTAINER(ad->applet), ad->display);

	display_events_init(ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet), "change_orient",
			   GTK_SIGNAL_FUNC(applet_change_orient), ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet), "change_size",
			   GTK_SIGNAL_FUNC(applet_change_size), ad);

	gtk_signal_connect(GTK_OBJECT(ad->applet), "change_background",
			   GTK_SIGNAL_FUNC(applet_do_draw), ad);

	panel_applet_setup_menu_from_file(applet, NULL, "GNOME_ClockmailApplet.xml",
					  NULL, applet_menu_verbs, ad);

	ad->tooltips = gtk_tooltips_new();
	g_object_ref(ad->tooltips);
	gtk_object_sink(GTK_OBJECT(ad->tooltips));

	gtk_widget_show(GTK_WIDGET(ad->applet));

	reload_skin(ad);

	update_display(ad);

	ad->update_timeout_id = gtk_timeout_add(1000, (GtkFunction)update_display, ad);
	
	return TRUE;
}

static gboolean applet_factory(PanelApplet *applet, const gchar *iid, gpointer data)
{
	gboolean retval = FALSE;

	if (strcmp(iid, "OAFIID:GNOME_ClockmailApplet") == 0)
		{
		retval = applet_widget_new(applet);
		}

	return retval;
}

PANEL_APPLET_BONOBO_FACTORY("OAFIID:GNOME_ClockmailApplet_Factory", PANEL_TYPE_APPLET,
			    "Clockmail-Applet", "0", applet_factory, NULL)

